(* This is the grammar of Oberon(-1) as distributed from Zurich, with two changes
    that make it fitting Depot4's parsing method
    Oberon programs containing, e.g., a FOR statemant will not be accepted

   Terminals, such as ident, string, etc. are taken from Dp4Stdlex.

 Dp4.Translate */n  Dp4.SetRoot module                                   *)

MODULE module 
	module='MODULE' ident ';' [ImportList ]DeclarationSequence ['BEGIN' 
	StatementSequence ]'END' ident '.' .
	ImportList='IMPORT' import {',' import }';' .
	import=ident [':=' ident ].
	identdef=ident ['*' ].
	qualident=('END'|'ELSE'|'ELSIF'|'UNTIL' |) !c=5; ident ['.' ident] .  (* <====== *)
	designator=qualident {'.' ident | '[' ExpList ']' | '(' qualident ')' | '^' }.
	ExpList=expression {',' expression }.
	ActualParameters='(' [ExpList ]')' .
END module
MODULE DeclarationSequence
	ConstantDeclaration=identdef '=' ConstExpression .
	ConstExpression=expression .
	TypeDeclaration=identdef '=' type .
	type=ArrayType | RecordType | PointerType | ProcedureType | qualident . (* <====== *)
	ArrayType='ARRAY' length {',' length }'OF' type .
	length=ConstExpression .
	RecordType='RECORD' ['(' BaseType ')' ]FieldListSequence 'END' .
	BaseType=qualident .
	FieldListSequence=FieldList {';' FieldList }.
	FieldList=[IdentList ':' type ].
	IdentList=identdef {',' identdef }.
	PointerType='POINTER' 'TO' type .
	ProcedureType='PROCEDURE' [FormalParameters ].
	VariableDeclaration=IdentList ':' type .
	ProcedureDeclaration=ProcedureHeading ';' ProcedureBody ident .
	ProcedureHeading='PROCEDURE' ['*' ]identdef [FormalParameters ].
	ProcedureBody=DeclarationSequence ['BEGIN' StatementSequence ]'END' .
	ForwardDeclaration='PROCEDURE' '^' ident ['*' ][FormalParameters ].
	DeclarationSequence={'CONST' {ConstantDeclaration ';' }| 'TYPE' {TypeDeclaration ';' }
    | 'VAR' {VariableDeclaration ';' }}{ProcedureDeclaration ';' | ForwardDeclaration ';' }.
	FormalParameters='(' [FPSection {';' FPSection }]')' [':' qualident ].
	FPSection=['VAR' ]ident {',' ident }':' FormalType .
	FormalType={'ARRAY' 'OF' }(qualident | ProcedureType ).
END DeclarationSequence
MODULE StatementSequence
	statement=[IfStatement | CaseStatement | WhileStatement | RepeatStatement | LoopStatement
   | WithStatement | 'EXIT' 
   | 'RETURN' [ expression ]
   | assignment | ProcedureCall].
	assignment=designator ':=' !! expression .
	ProcedureCall=  designator [ActualParameters ].
	StatementSequence=statement {';' !! statement }.
	IfStatement='IF' expression 'THEN' StatementSequence {'ELSIF' expression 'THEN'
   	StatementSequence }['ELSE' StatementSequence ]'END' .
	CaseStatement='CASE' expression 'OF' case {'|' case }['ELSE' 
	StatementSequence ]'END' .
	case=[CaseLabelList ':' StatementSequence ].
	CaseLabelList=CaseLabels {',' CaseLabels }.
	CaseLabels=ConstExpression ['..' ConstExpression ].
	WhileStatement='WHILE' expression 'DO' StatementSequence 'END' .
	RepeatStatement='REPEAT' StatementSequence 'UNTIL' expression .
	LoopStatement='LOOP' StatementSequence 'END' .
	WithStatement='WITH' qualident ':' qualident 'DO' StatementSequence 'END' .
END StatementSequence
MODULE expression
	expression=SimpleExpression [relation SimpleExpression ].
	SimpleExpression=['+' | '-' ]term {AddOperator term }.
	AddOperator='+' | '-' | 'OR' .
	term=factor {MulOperator factor }.
	MulOperator='*' | '/' | 'DIV' | 'MOD' | '&' .
	factor=CharConstant | number | string | 'NIL' | set | designator [ActualParameters ]
       | '(' expression ')' | '~' factor .
	set='{' [element {',' element }]'}' .
	element=expression ['..' expression ].
	relation='=' | '#' | '<=' | '<' | '>=' | '>' | 'IN' | 'IS' .
END expression
